@@ -10,7 +10,7 @@ REQUIRED_INTLTOOL_VERSION=0.35
 USE_COMMON_DOC_BUILD=yes
 
 (test -f $srcdir/configure.ac \
-  && test -d $srcdir/cddb-slave2) || {
+  && test -d $srcdir/gnome-volume-control) || {
     echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
     echo " top-level $PKG_NAME directory"
     exit 1
@@ -21,4 +21,4 @@ which gnome-autogen.sh || {
     exit 1
 }
 
-. gnome-autogen.sh --enable-shave "$@"
+. gnome-autogen.sh "$@"
